function [tpvals,t,tp,beta,Fpvals,F,Fp,tpvalcorr,bonf,fdr] = micro_func_model(par,conf)
% Do microstructure - function modeling for all homotopic pairs (node). For
% a given white matter tract, microstructure is extracted from the tbss
% skeleton voxels for each subject. This matrix is used to predict
% functional connectivity between the homotopic pair connected by the white
% matter tract. A principal components analysis is first performed on the
% microstructure matrix to reduce its dimensionality. The k=30 principal
% components are fed into a GLM to estimate homotopic functional
% connectivty. Then a permutation test is performed on the GLM to assess
% its significance in terms of a t-statistic (for each regressor) and an
% F-statistic for overall model performance.
%
% Prior to running this script:
%
% Functional connectivity is estimated between all homotopic region pairs
% for each subject individually and stored in a matrix (subs x nodes).
% (using rfMRI_preproccesing.m)
%
% All microstructure data is derived from the tbss skeleton voxels for each
% subject and stored in a matrix (subs x tbssvoxels).
% (using dMRI_get_micro.m)
% 
% Also, input your own created list of confound variables (subs x 
% confounds). 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Input:    par     - Microstructure metric to use, choose: 'MD','MO','FA',
%                     'ICVF','ISOVF', 'OD' or 'Multi', the latter combining
%                     all microstructure metrics into a multi-modal model
%           conf    - Confound variables for each subject (age, age^2, sex,
%                     age*sex, age^2*sex, resting-state fMRI head motion 
%                     and head size). The confound matrix (subs x conf)
%                     should be created by yourself as we cannot subject
%                     specific Biobank data here. 
%
%
% Outputs: tpvals    - p-value matrix for each regressor (node x k)
%          t         - t-statistic for each regressor (node x k)
%          tp        - Permuted t-statistics for each regressor (node x k x
%                      permutations)
%          beta      - Regression coefficients (node x k)
%          Fpvals    - p-value for all F-statistics (node x 1)
%          F         - F-statistic for each GLM (node x 1)
%          Fp        - Permuted F-statistics (node x permutations)
%          tpvalcorr - Family wise error corrected p-values (t-statistics)
%          bonf      - Bonferroni threshold
%          fdr       - False discovery rate threshold
%
% Author: Jeroen Mollink, University of Oxford (mollink.jeroen@gmail.com)


% Add FSL and FSLNets matlab functions to path
addpath(sprintf('%s/etc/matlab',getenv('FSLDIR')));
addpath /vols/Data/km/jmollink/Toolboxes/FSLNets/;

% Number of homotopic region pairs
nnode=81;
% Number of principal components to use in the design matrix of the
% microstructure - function glm
k = 30; 
% Number of permutations for permutation testing
nperm = 100000;


% Pre-allocate t-statistics and f-statistics matrices
t = zeros(nnode,k); tpvals = t; beta = t;
tp = zeros(nnode,k,nperm);
F = zeros(nnode,1); Fpvals = F; 
Fp = zeros(nnode,nperm);

for node = 1:nnode
    
    % Get tract-specific skeleton voxels (estimates from tbss)
    [tbss,~,pxdm] = read_avw('~/Biobank/Data/dMRI/tbss_fdt.nii.gz');
    % FSL's standard white matter skeleton mask (of all tracts in brain)
    tbssall = read_avw('~/Biobank/Data/dMRI/TBSS_mask.nii.gz');
    
   
    % Reshape tbss to voxels x nodes and mask out non tbss voxels
    tbssvec = reshape(tbss,numel(tbssall),size(tbss,4)); tbssvec = tbssvec(tbssall(:)>0,:);
    
    % Use multi-modal microstructure model or single microstructure metric
    % model
    if strcmp(par,'Multi');
        % For the multi-modal glm, load all microstructure metrics, de-mean,
        % and normalise through the first singular vector. Each individual
        % microstructure matrix is then concatenated before a principal
        % components analysis is performed
        
        % Microstructural metrics
        pars = {'FA','OD','ISOVF','ICVF','MD','MO'};
        Uall=[];
        for i=1:numel(pars)
            % Load microstructural parameter of interest from tbss skeleton voxels only
            P = load(sprintf('~/Biobank/Data/dMRI/%s_tbss_skel.mat',pars{i})); P = P.data';
            % Get dispersion from masked tbss voxels
            P = P(:,tbssvec(:,node)>0);
            % Demean and unit normalise
            P = nets_demean(P,1);
            
            % Perform singular value decomposition (principal components analysis)
            [~,sigma,~] = svd(P,'econ');
            Uall = [Uall,P./sigma(1)];
        end
        [U,~,~] = svd(Uall,'econ');
    else
        % Single modality glm, load microstructure metric specific by par.
        % De-mean, normalise variance and perform principal components analysis
        P = load(sprintf('~/Biobank/Data/dMRI/dMRI/%s_tbss_skel.mat',par)); P = P.data';
        % Get dispersion from masked tbss voxels
        P = P(:,tbssvec(:,node)>0);
        % Demean and unit normalise
        P = nets_demean(P,1); P = bsxfun(@rdivide,P,sqrt(sum(P.^2)));
        
        % Perform singular value decomposition (principal components analysis)
        [U,~,~] = svd(P,'econ');
        
    end
    
    
    % Select first k principal components
    Un = U(:,1:k);
       
    
    % Load functional connectivity of homotopic pairs
    FC = load('~/Biobank/Data/rfMRI/ICA/NetMats/homotopic_FC.mat'); FC = FC.homo;
    
    % Select functional connectivity of the homotopic pair of interest (node)
    Y = FC(:,node); Y = Y - mean(Y);
    % De-confound variables
    Y = Y - conf * (pinv(conf)*Y);
    X = Un - conf * (pinv(conf)*Un);
    
    % Permutation test of microstructure - function model
    [tpvals(node,:),t(node,:),tp(node,:,:),beta(node,:),...
        Fpvals(node),F(node),Fp(node,:)] = permutation(Y,X,nperm);
end

%%%% Family wise error correction to account for multiple comparison %%%%
% Max t-statistic distribution, across microstructural regressors and 
% homotopic pairs
maxT = squeeze(max(max(tp,[],1),[],2));

tpvalcorr = sum(repmat(maxT(:)',[nnode*k,1]) > repmat(abs(t(:)),[1,nperm]),2)./nperm;
tpvalcorr(tpvalcorr==0)=1/nperm;
% Corrected pvals, significance can be tested with an ?=0.05 threshold
tpvalcorr = reshape(tpvalcorr,[nnode,k]);


% Bonferroni threshold for significance (can be applied to tpvals)
bonf = 0.05/(nnode*k); 
% False Discovery Rate threshold for significance (can be applied to tpvals)
fdr = FDR(tpvals(:),0.05);




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%% SUPPORTING FUNCTIONS %%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [tpval,tstat,tstatsp,beta,Fpval,Fstat,Fstatsp] = permutation(Y,X,nperm)
% Get dimensions of design matrix
[ndata,nbeta] = size(X);

% Contrast of reduced model for F-test
c = [1 zeros(1,nbeta)];

% Add row of ones to first column of design matrix
desmat = [ones(ndata,1),X];

% Get beta, tstats of non-permuted (original) glm
[beta,~,stats] = ttest_glm(desmat,Y);
beta=beta(2:end); tstat = stats.t(2:end)';

% Get F-statis of non-permuted (original) glm
[~,Fstat] = ftest_glm(desmat,Y,c);

% Pre-allocate permuted t-statistics and F-statistics
tstatsp = zeros(nperm,nbeta); Fstatsp = zeros(nperm,1);
for i=1:nperm % Generate null-distribution
    % Shuffled indices
    ind = randperm(ndata);
    desmat_perm = desmat(ind,:);
    
    %%% At some point, merge ttest_glm and ftest_glm into one function
    % T-statistic glm for permuted design matrix
    [~,~,statsp] = ttest_glm(desmat_perm,Y);
    tstatsp(i,:) = statsp.t(2:end);
    
    % F-statistic glm for permuted design matrix
    [~,Fstatsp(i)] = ftest_glm(desmat_perm,Y,c);
    
    % Output progress
    if ~mod(i,1000); fprintf('Permutation %d\n',i); end
end

% p-value for each t-statistic (beta)
tstatsp=abs(tstatsp);
tpval = sum(tstatsp>repmat(abs(tstat),[nperm,1]))/nperm;
tpval(tpval==0) = 1./nperm;

% p-value for F-statistic
Fpval = sum(Fstatsp>repmat(Fstat,[nperm,1]))/nperm;
Fpval(Fpval==0) = 1./nperm;





function [cope,varcope,stats] = ttest_glm(x,y,c)
% [cope,varcope,stats] = fsl_glm(x,y,[c=Identity])
% stats.{t,p,z,dof}
%
% S. Jbabdi 03/14

if(nargin<3)
    if(size(x,2)<100)
        c = eye(size(x,2));
    else
        c = 1;
    end
end

% pseudo-inverse
[u,s,v] = svd(x,0);
s = diag(s);
tol = max(size(x)) * eps(norm(s,inf));
r1 = sum(s > tol)+1;
v(:,r1:end) = [];
u(:,r1:end) = [];
s(r1:end) = [];
s = 1./s(:);
invx = bsxfun(@times,v,s.')*u';
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


beta = invx*y;
cope = c*beta;
r    = y-x*beta;
dof  = size(r,1)-rank(x);

sigma_sq = sum(r.^2)/dof;

invxx   = bsxfun(@times,v,s.^2')*v';
varcope = diag((c*invxx*c')) * sigma_sq;
t       = cope./sqrt(varcope);
% p       = tcdf(t,dof); % 1-sided p-value for t-stat
t(isnan(t))=0;
stats.t   = t;
% stats.p   = p;
stats.dof = dof;
stats.z   = (2^0.5)*erfinv(1-2*betainc(dof./(dof+t.^2),dof/2,1/2)/2);
stats.z   = stats.z .* sign(t);
stats.ss  = sigma_sq;


function [betafull,F,pF] = ftest_glm(x,y,c)
%function [cope,varcope,stats] = fsl_glmjeroen(x,y,c)
% [cope,varcope,stats] = fsl_glm(x,y,[c=Identity])
% stats.{t,p,z,dof}
%
% Jeroen Mollink


% c: F-contrast
n=size(x,2);
C = zeros(n); C(1:(n+1):end)=c;

% pseudo-inverse full model
[u,s,v] = svd(x,0);
s = diag(s);
tol = max(size(x)) * eps(norm(s,inf));
r1 = sum(s > tol)+1;
v(:,r1:end) = [];
u(:,r1:end) = [];
s(r1:end) = [];
s = 1./s(:);
invx = bsxfun(@times,v,s.')*u';
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

betafull = invx*y;
rfull    = y-x*betafull;

% pseudo-inverse reduced model
[u,s,v] = svd(x*C,0);
s = diag(s);
tol = max(size(x)) * eps(norm(s,inf));
r1 = sum(s > tol)+1;
v(:,r1:end) = [];
u(:,r1:end) = [];
s(r1:end) = [];
s = 1./s(:);
invx = bsxfun(@times,v,s.')*u';
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

betareduce = invx*y;
rreduce   = y-(x*C)*betareduce;

% Degrees of freedom
dofF = size(rfull,1)-rank(x);
% Variance
sigma_sq = sum(rfull.^2);

% Degrees of freedom reduced model
dofR = size(rfull,1)-sum(c);
% Variance of reduced model
sigma_sq_red = sum(rreduce.^2);

% F-statistic
F = ((sigma_sq_red - sigma_sq)/(dofR-dofF)) ./ (sigma_sq/dofF);
pF = 1 - fcdf(F,dofR-dofF,dofF);


function [pID,pN] = FDR(p,q)
% FORMAT [pID,pN] = FDR(p,q)
%
% p   - vector of p-values
% q   - False Discovery Rate level
%
% pID - p-value threshold based on independence or positive dependence
% pN  - Nonparametric p-value threshold
%______________________________________________________________________________
% $Id: FDR.m,v 2.1 2010/08/05 14:34:19 nichols Exp $


p = p(isfinite(p));  % Toss NaN's
p = sort(p(:));
V = length(p);
I = (1:V)';

cVID = 1;
cVN = sum(1./(1:V));

pID = p(max(find(p<=I/V*q/cVID)));
if isempty(pID), pID=0; end
pN = p(max(find(p<=I/V*q/cVN)));
if isempty(pN), pN=0; end
